@@ -1,5 +1,5 @@
 #
-# $Id: Layer.pm 365 2015-01-28 19:04:14Z gomor $
+# $Id: Layer.pm,v 978fb2207025 2017/05/07 12:56:42 gomor $
 #
 package Net::Frame::Layer;
 use strict;
@@ -101,6 +101,8 @@ use Socket qw(:DEFAULT AF_INET);
 sub _setInet6Sub {
    no strict 'refs';
 
+   my $inetp_found = 0;
+
    # Check Socket against some IPv6 functions and constants.
    eval {
       require Socket;
@@ -110,19 +112,37 @@ sub _setInet6Sub {
       *{__PACKAGE__.'::_inet_pton'} = \&Socket::inet_pton;
       *{__PACKAGE__.'::_inet_ntop'} = \&Socket::inet_ntop;
 
-      return 1;
+      $inetp_found = 1;
    }
 
    # Fallback to Socket6
-   eval {
-      require Socket6;
-      Socket6->import(qw(AF_INET6 inet_pton inet_ntop));
-   };
-   if (! $@) {   # Socket6 has support for required functions and constants.
-      *{__PACKAGE__.'::_inet_pton'} = \&Socket6::inet_pton;
-      *{__PACKAGE__.'::_inet_ntop'} = \&Socket6::inet_ntop;
+   if (! $inetp_found) {
+      eval {
+         require Socket6;
+         Socket6->import(qw(AF_INET6 inet_pton inet_ntop));
+      };
+      if (! $@) {   # Socket6 has support for required functions and constants.
+         *{__PACKAGE__.'::_inet_pton'} = \&Socket6::inet_pton;
+         *{__PACKAGE__.'::_inet_ntop'} = \&Socket6::inet_ntop;
+      }
+   }
 
-      return 1;
+   # Unfortunately, we have to test if inet_ntop()/inet_pton() works (i.e., are implemented)
+   # If no support for inet_ntop/inet_pton, we branch to fake functions.
+   if ($inetp_found) {
+      eval {
+         # inet_pton() may exist, but die with:
+         # inet_pton not implemented on this architecture
+         _inet_pton(AF_INET6(), "::1");
+      };
+      if ($@) {
+         print "[!] inet_pton support: $@\n";
+         *{__PACKAGE__.'::_inet_pton'} = sub { return 0; };
+         *{__PACKAGE__.'::_inet_ntop'} = sub { return 0; };
+      }
+      else {
+         return 1;   # OK
+      }
    }
 
    die("[-] Net::Frame: inet_pton/inet_ntop: not supported by Socket nor Socket6: ".
@@ -132,6 +152,8 @@ sub _setInet6Sub {
 sub _setGetAddressSub {
    no strict 'refs';
 
+   my $getaddr_found = 0;
+
    # Check Socket against some IPv6 functions and constants.
    eval {
       require Socket;
@@ -169,31 +191,51 @@ sub _setGetAddressSub {
          }
       };
 
-      return 1;
+      $getaddr_found = 1;
    }
 
    # Fallback to Socket6
-   eval {
-      require Socket6;
-      Socket6->import(qw(getaddrinfo getnameinfo AF_INET6));
-   };
-   if (! $@) {   # Socket6 has support for required functions and constants.
-      *{__PACKAGE__.'::_getAddress'} = sub {
-         my ($name) = @_;
-
-         #print STDERR "*** Fallback to Socket6 support\n";
-
-         my @res = Socket6::getaddrinfo($name, '', Socket6::AF_INET6(), SOCK_STREAM);
-         if (@res >= 5) {
-            my ($ipv6) = Socket6::getnameinfo(
-               $res[3], Socket6::NI_NUMERICHOST() | Socket6::NI_NUMERICSERV()
-            );
-
-            return $ipv6;
-         }
+   if (! $getaddr_found) {
+      eval {
+         require Socket6;
+         Socket6->import(qw(getaddrinfo getnameinfo AF_INET6));
       };
+      if (! $@) {   # Socket6 has support for required functions and constants.
+         *{__PACKAGE__.'::_getAddress'} = sub {
+            my ($name) = @_;
 
-      return 1;
+            #print STDERR "*** Fallback to Socket6 support\n";
+
+            my @res = Socket6::getaddrinfo($name, '', Socket6::AF_INET6(), SOCK_STREAM);
+            if (@res >= 5) {
+               my ($ipv6) = Socket6::getnameinfo(
+                  $res[3], Socket6::NI_NUMERICHOST() | Socket6::NI_NUMERICSERV()
+               );
+
+               return $ipv6;
+            }
+         };
+      }
+
+      $getaddr_found = 1;
+   }
+
+   # Unfortunately, we have to test if INET6 family is supported
+   # If no support, we branch to fake functions.
+   if ($getaddr_found) {
+      eval {
+         # getaddrinfo() may exist, but die with:
+         # getaddrinfo: ai_family not supported
+         _getAddress("::1");
+      };
+      if ($@) {
+         print "[!] getaddrinfo support: $@\n";
+         *{__PACKAGE__.'::_getAddress'} = sub { return 0; };
+         *{__PACKAGE__.'::_getAddress'} = sub { return 0; };
+      }
+      else {
+         return 1;   # OK
+      }
    }
 
    die("[-] Net::Frame: getaddrinfo/getnameinfo: not supported by Socket nor Socket6: ".
@@ -462,7 +504,7 @@ Patrice E<lt>GomoRE<gt> Auffret
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006-2015, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2006-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of the Artistic license.
 See LICENSE.Artistic file in the source distribution archive.
